import os

from charmhelpers.core import (
    hookenv,
    host,
    )
from charmhelpers.core.services.base import ServiceManager
from charmhelpers.core.services import helpers

import actions
import turnip_helpers


def render_env_template(config, name):
    return helpers.render_template(
        source='envs/{}.j2'.format(name),
        target='{}/{}'.format(os.path.join(config['base_dir'], 'envs'), name),
        owner=config['code_user'],
        group=config['code_group'])


class RequiresLiveTurnip:
    def __bool__(self):
        return self.is_live()

    __nonzero__ = __bool__

    def is_live(self):
        return host.service_available("turnip")


def manage():
    if hookenv.hook_name() in ('install', 'upgrade-charm'):
        actions.execd_preinstall('turnip')
        actions.install_packages('turnip')
        actions.create_users('turnip')
        actions.unpack_source('turnip')
        actions.install_python_packages('turnip')

    extra_requirements = []

    if hookenv.config()['storage_external']:
        # Request a volume from the storage subordinate, and ensure the
        # service doesn't try to start until it's present.
        for relid in hookenv.relation_ids('data'):
            hookenv.relation_set(relid, mountpoint=actions.DATA_DIR)
        extra_requirements.append(turnip_helpers.TurnipStorageRelation())

    config = hookenv.config()
    manager = ServiceManager([
        {
            'service': 'turnip',
            'ports': actions.get_ports(),
            'provided_data': [turnip_helpers.TurnipApiRelation()],
            'required_data': [config] + extra_requirements,
            'data_ready': [
                actions.ensure_data_dir_writable,
                helpers.render_template(
                    source='turnip.conf.j2',
                    target='/etc/init/turnip.conf',
                    owner=config['code_user'],
                    group=config['code_group'],
                    perms=0o644),
                render_env_template(config, 'REPO_STORE'),
                render_env_template(config, 'TURNIP_LOG_DIR'),
                render_env_template(config, 'VIRTINFO_ENDPOINT'),
                render_env_template(config, 'AUTHENTICATION_ENDPOINT'),
                render_env_template(config, 'PRIVATE_KEY_PATH'),
                render_env_template(config, 'PUBLIC_KEY_PATH'),
                actions.publish_wsgi_relations,
            ],
        },
        {
            'service': 'turnip-monitoring',
            'required_data': [config],
            'data_ready': [
                helpers.render_template(
                    source='turnip-nrpe.j2',
                    target='/etc/nagios/nrpe.d/check_{}.cfg'.format(
                        hookenv.local_unit().replace('/', '-'),
                    )
                ),
                helpers.render_template(
                    source='turnip-nagios.j2',
                    target='/var/lib/nagios/export/service__{}-{}.cfg'.format(
                        config['nagios_context'],
                        hookenv.local_unit().replace('/', '-'),
                    )
                ),
            ],
            'data_lost': [
                helpers.render_template(
                    source='turnip-nrpe.j2',
                    target='/etc/nagios/nrpe.d/check_{}.cfg'.format(
                        hookenv.local_unit().replace('/', '-'),
                    )
                ),
                helpers.render_template(
                    source='turnip-nagios.j2',
                    target='/var/lib/nagios/export/service__{}-{}.cfg'.format(
                        config['nagios_context'],
                        hookenv.local_unit().replace('/', '-'),
                    )
                ),
            ],
        },
        {
            'service': 'turnip-post',
            'required_data': [RequiresLiveTurnip()],
            'data_ready': [
                actions.publish_website_relations,
            ],
        },
    ])
    manager.manage()
